/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : SCPMutationWD.cc                                         */
/*                                                                       */
/*       Date : March 5, 1998                                            */
/* Updated at : March 5, 1998                                            */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <fstream.h>

#include <deffunc.h>
#include <MutationOperator.h>

#include "SCPMutationWD.h"
#include "SCPChromosomes.h"
#include "SetCoverProblem.h"

/*************************************************************************/
/*************************************************************************/
SCPMutationWD::SCPMutationWD() {  debugOn(); }

/*************************************************************************/
/*************************************************************************/
SCPMutationWD::~SCPMutationWD()
{  cout << " de allocate SCPMutation " << endl; }

/*************************************************************************/
/*************************************************************************/
Chromosomes * SCPMutationWD::apply(Chromosomes *_or)
{
   SCPChromosomes *parent = (SCPChromosomes *) _or;
   SetCoverProblem *myProblem = parent->getProblem();
   SCPChromosomes *ret  = new SCPChromosomes(myProblem);
   int NumOfRows = myProblem->getNumOfRows();
   int NumOfCols = parent->getNumOfColumns();
   int *weights  = parent->getWeights();
   int *w = new int[NumOfRows];
   int i, j, k;
   double sumw, sum, r;
   Column *c;

   // Calculate weights and sum 
   for(i=0;i<NumOfRows;i++) { w[i] = weights[i]-1; sumw+=w[i]; }

   //
   k = 0; sum=w[k];
   r = 0.01 * (rand()%101);
   while( sum>r ) { k=(k+1)%NumOfRows; sum+=w[i]; }

   // get columns covering row k
   tSet<int> k_cols ;
   tSet<int> cols = parent->getColumns();
   for(i=0;i<parent->getNumOfColumns();i++)
      if( (c=parent->getColumnWithIndx(i))!=NULL && c->test(k) )
	  k_cols.addElement(i);

   *ret = *parent;

   // if some
   if(k_cols.getSize())
   {
      // randomly find the column which has to be deleted
      int k_del = rand() % k_cols.getSize();
      int k_abon;

      // get the order index of this column
      if( k_cols.getElementAtSafe(k_del,k_del) )
      {
          // get the column index of this column
          k_abon = parent->getColNumWithIndex(k_del);

          // delete the k_del th column in the solution
	  ret->deleteColumnWithIndex(k_del);

          // prepare abondon list
          k_cols.clearAll();
	  k_cols.addElement(k_abon);

          // restore the solution
	  repair(ret,&k_cols);

          // reduce the solution
	  ret->reduce();

      }// end of if

   }// end of if

   return ret;

}// end of apply()

/*************************************************************************/
/*************************************************************************/
ChromosomesBag SCPMutationWD::apply( Chromosomes *_or,
				     Chromosomes *_fr, float percent)
{
   ChromosomesBag ret(2);
   Chromosomes *ret1 = apply(_or);

   if(ret1!=NULL) ret.addChromosomes(ret1);

   return ret;

}// end of aply()

/*************************************************************************/
/*************************************************************************/
void SCPMutationWD::repair( SCPChromosomes *chr, tSet<int>* abondon)
{
   chr->restoreRandom(abondon);

}// end of aply()
